Files
project_6/cccl_upstream/cudax/examples/vector_add.cu
EngineX CI 56fd68e7dd [INFRA] Import NVIDIA/CCCL upstream as optimization reference library
CCCL (CUDA C++ Core Libraries) provides:
- CUB: device/block/warp-level GPU primitives (reduce, scan, sort, topk)
- Thrust: high-level parallel algorithms (transform_reduce, sort, scan)
- libcudacxx: CUDA C++ standard library (atomics, barriers, memory)
- cudax: experimental features (memory resources, allocators)
- Tuning policies: per-SM hardware-specific algorithm parameters

Competition optimization vectors mapped to CCCL:
- Output TPS (83% weight): warp_reduce, block_reduce, device_topk
- Input TPS (14% weight): device_scan, block_load, prefetch
- Cache TPS (3% weight): prefix caching strategy patterns
- Memory (0.9 util): pooled/cached/buddy allocators

Source: https://github.com/NVIDIA/cccl (shallow clone, HEAD only)
License: Apache-2.0
2026-07-30 09:35:51 +00:00

128 lines
4.0 KiB
Plaintext

/* Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* * Redistributions of source code must retain the above copyright
* notice, this list of conditions and the following disclaimer.
* * Redistributions in binary form must reproduce the above copyright
* notice, this list of conditions and the following disclaimer in the
* documentation and/or other materials provided with the distribution.
* * Neither the name of NVIDIA CORPORATION nor the names of its
* contributors may be used to endorse or promote products derived
* from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/**
* Vector addition: C = A + B.
*
* This sample is a very basic sample that implements element by element
* vector addition. It is the same as the sample illustrating Chapter 2
* of the programming guide with some additions like error checking.
*/
#include <cstdio>
// For the CUDA runtime routines (prefixed with "cuda_")
#include <cuda/std/span>
#include <cuda/experimental/launch.cuh>
#include <cuda/experimental/stream.cuh>
#include <cuda_runtime.h>
#include "vector.cuh"
namespace cudax = cuda::experimental;
using cudax::in;
using cudax::out;
/**
* CUDA Kernel Device code
*
* Computes the vector addition of A and B into C. The 3 vectors have the same
* number of elements numElements.
*/
__global__ void vectorAdd(cudax::span<const float> A, cudax::span<const float> B, cudax::span<float> C)
{
int i = static_cast<int>(blockDim.x * blockIdx.x + threadIdx.x);
if (i < A.size())
{
C[i] = A[i] + B[i] + 0.0f;
}
}
/**
* Host main routine
*/
int main()
try
{
// A CUDA stream on which to execute the vector addition kernel
cudax::stream stream(cuda::devices[0]);
// Print the vector length to be used, and compute its size
int numElements = 50000;
printf("[Vector addition of %d elements]\n", numElements);
// Allocate the host vectors
cudax::vector<float> A(numElements); // input
cudax::vector<float> B(numElements); // input
cudax::vector<float> C(numElements); // output
// Initialize the host input vectors
for (int i = 0; i < numElements; ++i)
{
A[i] = static_cast<float>(rand()) / (float) RAND_MAX;
B[i] = static_cast<float>(rand()) / (float) RAND_MAX;
}
// Define the kernel launch parameters
constexpr int threadsPerBlock = 256;
auto config = cuda::distribute<threadsPerBlock>(numElements);
// Launch the vectorAdd kernel
printf("CUDA kernel launch with %zu blocks of %d threads\n", cuda::block.count(cuda::grid, config), threadsPerBlock);
cudax::launch(stream, config, vectorAdd, in(A), in(B), out(C));
printf("waiting for the stream to finish\n");
stream.sync();
printf("verifying the results\n");
// Verify that the result vector is correct
for (int i = 0; i < numElements; ++i)
{
if (fabs(A[i] + B[i] - C[i]) > 1e-5)
{
fprintf(stderr, "Result verification failed at element %d!\n", i);
exit(EXIT_FAILURE);
}
}
printf("Test PASSED\n");
printf("Done\n");
return 0;
}
catch (const std::exception& e)
{
printf("caught an exception: \"%s\"\n", e.what());
}
catch (...)
{
printf("caught an unknown exception\n");
}